/*
    This file is part of AutoQuad.

    AutoQuad is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    AutoQuad is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with AutoQuad.  If not, see <http://www.gnu.org/licenses/>.

    Copyright © 2011-2014  Bill Nesbitt
*/

#ifndef _spektrum_h
#define _spektrum_h

#include "radio.h"

#define SPEKTRUM_BAUD		115200
#define SPEKTRUM_RXBUF_SIZE	64

typedef struct {
    uint32_t lastCharReceived;
    uint8_t rawBuf[16];
    uint8_t state;
} spektrumStruct_t;

//  Spektrum X-Plus channel decoding
//  Channel numbers 0 to 11 are normal, with data resolution 10(1024 steps) or 11bit (2048 steps).  
//  Channel 12 always carries the X-Plus channel data when X-Plus is enabled. The data resolution of channel 
//  12 to 17 is 9bit (512 steps).
//  Please notice that channel 11 and 12 are duplicates of channel X+1 (10) and channel X+2 (11).
//
//  Channel transfer structure:
//  packet 0 -> 1 5 2 4 6 10 12
//  packet 1 -> 0 7 3 8 9 11 12
//
//  each 16bit data value is defined as:
//  10bit:
//  a 0 bbbb cccccccccc
//  11bit:
//  a bbbb  ccccccccccc
//
//  a:    always 0, 1 only for 1st channel in 2nd data packet
//  bbbb: the channel number of this packet
//  cc..: 10 or 11bit channel data (MSB 1st)
//
//  The channel data will be 0xffff if the specific channel slot is unused
//
//  Each 16bit X-Plus data value is defined as:
//
//  a bbbb cc ddddddddd
//
//  a: always 0
//  bbbb: always 12, since channel 12 contains X-Plus channel data when X-Plus is enabled on the transmitter
//  cc: X-Plus channel number, lower 2 bits only. The third bit is defined by the packet number containing the data:
//    transferred via packet 0 --> channel# = 0b0cc
//    transferred via packet 1 --> channel# = 0b1cc
//    Channel number range during transfer is 0 to 7!
//
//  ddddddddd:  X-Plus channel data, 9 bits resolution (512), mid at 256
//              On a DX10t the value range at +/-150% (max trim added) is 0 to 511; with no trim value range is 85 (low) - 256 (mid) - 426 (high) 
//
//  Since there are two TX-frames containing X-Plus channel data (packet 0 and packet1) we have 2 X-Plus channels at the same frame rate.
//  The possible frame rates were 22/44/66/88ms (with reference to Spektrum DX18T Users Manual). Frame rate for the channels is selected 
//  based on (still unknown) priority scheme. One factor which increases priority is the amount of  change applied to the specific channel.
//
//  X-PLUS CHANNELS SHOULD BE USED FOR LOW-PRIO CONTROL (I.E. LIGHTS, GIMBAL, ETC) ONLY DUE TO FRAME RATE VARIATION!

extern void spektrumInit(radioInstance_t *r, USART_TypeDef *uart);
extern uint8_t spektrumCharIn(radioInstance_t *r, int c);

extern spektrumStruct_t spektrumData;

#endif
